#!/usr/bin/env python3
"""
SSDP Service Discoverer
Parameters:
    - IP: Network interface IP to listen on
Additional parameters in comments:
    - timeout: Discovery timeout in seconds
"""
import socket
import time
import argparse
from datetime import datetime

class SSDPDiscoverer:
    def __init__(self, interface_ip, timeout=5):
        self.interface_ip = interface_ip
        self.timeout = timeout
        self.SSDP_ADDR = '239.255.255.250'
        self.SSDP_PORT = 1900
        self.SSDP_MX = 1
        self.SSDP_ST = 'ssdp:all'
        
    def create_socket(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.bind((self.interface_ip, 0))
        return sock
        
    def create_msearch_request(self):
        return (
            'M-SEARCH * HTTP/1.1\r\n' +
            f'HOST: {self.SSDP_ADDR}:{self.SSDP_PORT}\r\n' +
            'MAN: "ssdp:discover"\r\n' +
            f'ST: {self.SSDP_ST}\r\n' +
            f'MX: {self.SSDP_MX}\r\n' +
            '\r\n'
        ).encode()
        
    def discover(self):
        print(f"Starting SSDP discovery on interface {self.interface_ip}")
        print(f"Listening for {self.timeout} seconds...\n")
        
        sock = self.create_socket()
        sock.settimeout(1)
        
        # Send M-SEARCH request
        sock.sendto(self.create_msearch_request(), (self.SSDP_ADDR, self.SSDP_PORT))
        
        start_time = time.time()
        devices = {}
        
        try:
            while time.time() - start_time < self.timeout:
                try:
                    data, addr = sock.recvfrom(1024)
                    response = data.decode('utf-8')
                    
                    # Parse response headers
                    headers = {}
                    for line in response.split('\r\n'):
                        if ':' in line:
                            key, value = line.split(':', 1)
                            headers[key.strip().lower()] = value.strip()
                    
                    if addr[0] not in devices:
                        devices[addr[0]] = headers
                        print(f"\nFound device at {addr[0]}:")
                        if 'server' in headers:
                            print(f"Server: {headers['server']}")
                        if 'location' in headers:
                            print(f"Location: {headers['location']}")
                        if 'st' in headers:
                            print(f"Service Type: {headers['st']}")
                        print("-" * 50)
                        
                except socket.timeout:
                    continue
                    
        except KeyboardInterrupt:
            print("\nDiscovery stopped by user")
        finally:
            sock.close()
            
        print(f"\nDiscovery complete. Found {len(devices)} unique devices.")
        print(f"Timestamp: {datetime.now()}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="SSDP Service Discoverer")
    parser.add_argument("ip", help="Network interface IP to listen on")
    parser.add_argument("--timeout", type=int, default=5,
                      help="Discovery timeout in seconds")
    args = parser.parse_args()
    
    discoverer = SSDPDiscoverer(args.ip, args.timeout)
    discoverer.discover()
